﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Core;
using HeuristicLab.DataPreprocessing.Filter;
using HeuristicLab.Collections;

namespace HeuristicLab.DataPreprocessing.Views
{
  [View("CheckedFilterCollection View")]
  [Content(typeof(FilterContent), false)]
  public partial class FilterView : ItemView
  {
    private CheckedItemCollectionView<IFilter> filterView;

    public FilterView()
    {
      InitializeComponent();
      tbTotal.Text = "0";
      tbFiltered.Text = "0";
      tbPercentage.Text = "0%";
      filterView = new CheckedItemCollectionView<IFilter>();
      filterView.Dock = DockStyle.Fill;
      filterView.Content = new CheckedItemCollection<IFilter>();
      filterView.Content.ItemsAdded += Content_ItemsAdded;
      filterView.Content.ItemsRemoved += Content_ItemsRemoved;
      filterView.Content.CheckedItemsChanged += Content_CheckedItemsChanged;
      groupBoxFilter.Controls.Add(filterView);
    }

    public new FilterContent Content
    {
      get { return (FilterContent)base.Content; }
      set { base.Content = value; tbTotal.Text = Content.FilterLogic.PreprocessingData.Rows.ToString(); }
    }

    private void Content_CheckedItemsChanged(object sender, Collections.CollectionItemsChangedEventArgs<IFilter> e)
    {
      //change active state of filters
      foreach (IFilter filter in e.Items)
      {
        filter.Active = !filter.Active;
      }
      UpdateFilterInfo();
      Refresh();
      ResumeRepaint(true);
    }

    private void UpdateFilterInfo()
    {
      List<IFilter> filters = filterView.ItemCollection.ToList<IFilter>();
      bool[] result = Content.FilterLogic.Preview(filters);
      int filteredCnt = 0;
      for (int i = 0; i < result.Length; i++)
      {
        if (result[i] == true)
          filteredCnt++;
      }
      tbFiltered.Text = filteredCnt.ToString();
      double percentage = filteredCnt * 100 / result.Length;
      tbPercentage.Text = percentage.ToString() + "%";
      tbTotal.Text = result.Length.ToString();
    }

    private void applyFilterButton_Click(object sender, EventArgs e)
    {
      //todo: inform Logic for apply Filter
    }

    //whenever a new filter is added the preprocessing data is set to the filter
    private void Content_ItemsAdded(object sender, Collections.CollectionItemsChangedEventArgs<IFilter> e)
    {
      foreach (IFilter filter in e.Items)
      {
        filter.ConstrainedValue = Content.FilterLogic.PreprocessingData;
        filter.Active = true;
        filterView.Content.SetItemCheckedState(filter, false);
      }
      UpdateFilterInfo();
    }

    private void Content_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<IFilter> e)
    {
      UpdateFilterInfo();
    }

  }
}
